# virtual-server-lib.pl
# Common functions for Virtualmin

do '../web-lib.pl';
&init_config();
require '../ui-lib.pl';
use Time::Local;
%access = &get_module_acl();
if (!defined($access{'feature_unix'})) {
	$access{'feature_unix'} = 1;
	}
if (!defined($access{'feature_dir'})) {
	$access{'feature_dir'} = 1;
	}

if (!$done_virtual_server_lib_funcs) {
	do 'virtual-server-lib-funcs.pl';
	}

# Can this user only view mailboxes in one domain? If so, we use a special UI
$single_domain_mode = $access{'domains'} =~ /^\d+$/ &&
		      !$access{'edit'} && !$access{'create'} &&
		      !$access{'stop'} && !$access{'local'} ?
			$access{'domains'} : undef;

if (!&master_admin()) {
	# Allowed alias types are set by module config
	%can_alias_types = map { $_, 1 } split(/,/, $config{'alias_types'});
	}
else {
	# All types are allowed
	%can_alias_types = map { $_, 1 } (0 .. 11);
	}

$first_print = \&first_html_print;
$second_print = \&second_html_print;
$indent_print = \&indent_html_print;
$outdent_print = \&outdent_html_print;

# hlink(text, page, [module], [width], [height])
# This is an override for the standard hlink function which checks if the
# file really exists.
sub checked_hlink
{
local ($text, $page, $mod, $width, $height) = @_;
$mod ||= $module_name;
if (!-r &help_file($mod, $page)) {
	return $text;
	}
else {
	$width ||= $tconfig{'help_width'} || $gconfig{'help_width'} || 400;
	$height ||= $tconfig{'help_height'} || $gconfig{'help_height'} || 300;
	return "<a onClick='window.open(\"$gconfig{'webprefix'}/help.cgi/$mod/$_[1]\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=$width,height=$height,resizable=yes\"); return false' href=\"$gconfig{'webprefix'}/help.cgi/$mod/$_[1]\">$_[0]</a>";
	}
}

$virtualmin_pro = $module_info{'virtualmin'} eq 'pro' ? 1 : 0;
if (!$virtualmin_pro) {
	$config{'status'} = 0;
	$config{'spam'} = 0;
	$config{'virus'} = 0;
	$original_hlink = $main::{'hlink'} ||
			  $virtual_server::{'hlink'} ||
			  $gpl_virtual_server::{'hlink'};
	$main::{'hlink'} = \&checked_hlink;
	$virtual_server::{'hlink'} = \&checked_hlink;
	$gpl_virtual_server::{'hlink'} = \&checked_hlink;
	}

@used_webmin_modules = ( "acl", "apache", "bind8", "cron", "htaccess-htpasswd",
			 "init", "ldap-useradmin", "logrotate", "mailboxes",
			 "mount", "mysql", "net", "postfix", "postgresql",
			 "proc", "procmail", "qmailadmin", "quota", "sendmail",
			 "servers", "software",
			 $virtualmin_pro ? ( "spam", "status" ) : ( ),
			 "syslog", "useradmin", "usermin", "webalizer",
			 "webmin" );
@confplugins = split(/\s+/, $config{'plugins'});
@opt_features = ( 'dir', 'unix', 'mail', 'dns', 'web', 'webalizer', 'ssl',
		  'logrotate', 'mysql', 'postgres', 'ftp',
		  $virtualmin_pro ? ( 'spam', 'virus', 'status' ) : ( ),
		  'webmin' );
@vital_features = ( 'dir', 'unix' );
@features = ( @opt_features );
@backup_features = ( 'virtualmin', @features );
@opt_alias_features = ( 'dir', 'mail', 'dns', 'web' );
@opt_subdom_features = ( 'dir', 'dns', 'web', 'ssl' );
@alias_features = ( @opt_alias_features );
@database_features = ( 'mysql', 'postgres' );
@template_features = ( 'basic', 'limits', @features, 'virt',
		       $virtualmin_pro ? ( 'virtualmin' ) : ( ),
		       'plugins',
		       $virtualmin_pro ? ( 'scripts' ) : ( ) );
@template_features_effecting_webmin = ( 'web', 'webmin' );
foreach my $fname (@features, "virt") {
	if (!$done_feature_script{$fname} || $force_load_features) {
		do "$module_root_directory/feature-$fname.pl";
		}
	local $ifunc = "init_$fname";
	&$ifunc() if (defined(&$ifunc));
	}
@plugins = ( );
foreach my $pname (@confplugins) {
	if (&foreign_check($pname)) {
		&foreign_require($pname, "virtual_feature.pl");
		push(@plugins, $pname);
		}
	}
@feature_plugins = grep { &plugin_defined($_, "feature_setup") } @plugins;
@mail_plugins = grep { &plugin_defined($_, "mailbox_inputs") } @plugins;
@database_plugins = grep { &plugin_defined($_, "database_name") } @plugins;
@startstop_plugins = grep { &plugin_defined($_, "feature_startstop") } @plugins;
@backup_plugins = grep { &plugin_defined($_, "feature_backup") } @plugins;
@migration_types = $virtualmin_pro ? ( "cpanel" ) : ( );
foreach my $m (@migration_types) {
	do "$module_root_directory/migration-$m.pl";
	}
@allow_features = (@opt_features, "virt", @feature_plugins);
@startstop_features = ("web", "dns", "mail", "ftp", "mysql", "postgres");
@all_database_types = ( ($config{'mysql'} ? ("mysql") : ( )),
		        ($config{'postgres'} ? ("postgres") : ( )),
		        @database_plugins );
		       

$backup_cron_cmd = "$module_config_directory/backup.pl";
$bw_cron_cmd = "$module_config_directory/bw.pl";
$licence_cmd = "$module_config_directory/licence.pl";
$licence_status = "$module_config_directory/licence-status";
$quotas_cron_cmd = "$module_config_directory/quotas.pl";

$custom_fields_file = "$module_config_directory/custom-fields";

@scripts_directories = ( "$module_config_directory/scripts",
			 "$module_root_directory/scripts",
		       );
$script_log_directory = "$module_config_directory/scriptlog";
$scripts_unavail_file = "$module_config_directory/scriptsunavail";

@reseller_maxes = ("doms", "aliasdoms", "realdoms", "quota", "mailboxes", "aliases", "dbs", "bw");

@all_template_files = ( "domain-template", "subdomain-template",
			"user-template", "local-template", "bw-template",
			"warnbw-template", "framefwd-template",
			"update-template" );

$initial_users_dir = "$module_config_directory/initial";

@edit_limits = ('domain', 'users', 'aliases', 'dbs', 'scripts',
	        'ip', 'ssl', 'forward', 'admins', 'spam', 'backup',
		'disable', 'delete');

@virtualmin_backups = ( 'config', 'templates',
			$virtualmin_pro ? ( 'resellers' ) : ( ),
			'email', 'custom',
			$virtualmin_pro ? ( 'scripts' ) : ( ) );

@limit_types = ("mailboxlimit", "aliaslimit", "dbslimit", "domslimit",
            	"aliasdomslimit", "realdomslimit");

$bandwidth_dir = "$module_config_directory/bandwidth";
$plainpass_dir = "$module_config_directory/plainpass";

$template_scripts_dir = "$module_config_directory/template-scripts";

foreach my $lib ("scripts", "resellers", "admins", "simple", "s3") {
	if (-r "$module_root_directory/$lib-lib.pl") {
		do "$module_root_directory/$lib-lib.pl";
		}
	}

$domains_dir = "$module_config_directory/domains";
$templates_dir = "$module_config_directory/templates";
$domainnames_dir = "$module_config_directory/names";

